#!/usr/bin/env python3


"""Load a digital elevation model and apply it to an r5py.TransportNetwork."""


import rasterio

from .elevation_cost_function import ElevationCostFunction
from .file_storage import FileStorage
from ..util import WorkingCopy

import com.conveyal.analysis
import com.conveyal.r5


__all__ = ["ElevationModel"]


class ElevationModel:
    """Load a digital elevation model and apply it to an r5py.TransportNetwork."""

    def __init__(
        self,
        elevation_model,
        elevation_cost_function=ElevationCostFunction.TOBLER,
    ):
        """
        Load an elevation model.

        Arguments
        ---------
        elevation_model : str | pathlib.Path
            file path to a digital elevation model in TIF format,
            single-band, the value of which is the elevation in metres
        elevation_cost_function : r5py.ElevationCostFunction
            which algorithm to use to compute the added effort and travel time
            of slopes
        """
        elevation_model = WorkingCopy(elevation_model)
        elevation_model = self._convert_tiff_to_format_readable_by_r5(elevation_model)

        # instantiate an com.conveyal.file.FileStorage singleton
        com.conveyal.analysis.components.WorkerComponents.fileStorage = FileStorage()

        self._elevation_model = com.conveyal.r5.analyst.scenario.RasterCost()
        self._elevation_model.dataSourceId = f"{elevation_model.with_suffix('')}"
        self._elevation_model.costFunction = elevation_cost_function

    def apply_to(self, transport_network):
        """
        Add the costs associated with elevation traversal to a transport network.

        Arguments
        ---------
        transport_network : r5py.TransportNetwork
            The transport network to which to add slope costs
        """
        self._elevation_model.resolve(transport_network)
        self._elevation_model.apply(transport_network)

    @staticmethod
    def _convert_tiff_to_format_readable_by_r5(tiff):
        # javax.imagio does not allow all compression/predictor
        # combinations of TIFFs
        # to work around it, convert the input to a format known to work.

        input_tiff = tiff.with_stem(f".{tiff.stem}")
        output_tiff = tiff.with_suffix(".tif")
        tiff.rename(input_tiff)

        with rasterio.open(input_tiff) as source:
            metadata = source.profile
            metadata.update(
                {
                    "compress": "LZW",
                    "predictor": "2",
                }
            )
            with rasterio.open(output_tiff, "w", **metadata) as destination:
                destination.write(source.read())
        input_tiff.unlink()

        return output_tiff



#!/usr/bin/env python3


"""The elevation cost functions supported by R5 (Tobler, Minetti)."""


import enum

import jpype

from ..util import start_jvm

import com.conveyal.r5


__all__ = ["ElevationCostFunction"]


start_jvm()


class ElevationCostFunction(enum.Enum):
    """
    Elevation cost functions.

    TOBLER: Waldo Tobler’s hiking function, cf. https://en.wikipedia.org/wiki/Tobler%27s_hiking_function
    MINETTI: Minetti et al.’s perceived effort/energy consumption, cf.
    https://doi.org/10.1152/japplphysiol.01177.2001
    """

    @classmethod
    def _missing_(cls, value):
        value = str(value).upper()
        for member in cls:
            if value == member.value:
                return member
        return None

    TOBLER = "TOBLER"
    MINETTI = "MINETTI"


@jpype._jcustomizer.JConversion(
    "com.conveyal.r5.analyst.scenario.RasterCost.CostFunction",
    exact=ElevationCostFunction,
)
def _cast_LegMode(java_class, object_):
    return com.conveyal.r5.analyst.scenario.RasterCost.CostFunction.valueOf(
        object_.name
    )



#!/usr/bin/env python3


import pytest

import r5py


class TestElevationCostFunction:
    def test_some_weird_combinations(self):
        # not as a parametrized test as pytest has troubles with
        # functions as parametrized fixtures (tries to evaluate?)
        assert r5py.ElevationCostFunction("tobler") == r5py.ElevationCostFunction.TOBLER
        assert r5py.ElevationCostFunction("Tobler") == r5py.ElevationCostFunction.TOBLER
        assert r5py.ElevationCostFunction("tObLeR") == r5py.ElevationCostFunction.TOBLER
        assert r5py.ElevationCostFunction("TOBLER") == r5py.ElevationCostFunction.TOBLER
        assert (
            r5py.ElevationCostFunction("minetti") == r5py.ElevationCostFunction.MINETTI
        )
        assert (
            r5py.ElevationCostFunction("Minetti") == r5py.ElevationCostFunction.MINETTI
        )
        assert (
            r5py.ElevationCostFunction("mInEtTi") == r5py.ElevationCostFunction.MINETTI
        )
        assert (
            r5py.ElevationCostFunction("MINETTI") == r5py.ElevationCostFunction.MINETTI
        )

        with pytest.raises(ValueError):
            _ = r5py.ElevationCostFunction("FooBarInvalidCostFunction")



#!/usr/bin/env python3


"""Fixtures describing the expected routing results."""


import pytest

from .routing_results import ISOCHRONES_WALK


ISOCHRONES_WALK_BLAKE2B = "e5c0fe9d34b512363c876eec81439b2708a1171272dd721a40aefe502c6e0dc73811aab0f109a45ab83645b88dea38051112ede7fc4b99a1231d92e3b4492e11"
ISOCHRONES_WALK_BLAKE2S = (
    "3d46b490841df409cbb962e25d4201bda0eea5b825fc1181cb0babfddf457299"
)
ISOCHRONES_WALK_SHA256 = (
    "1d899da67faed2f49a0dd870b1dc33b4c7bd695b18eafc08640a662b442e935c"
)


@pytest.fixture
def file_digest_test_file_as_pathlib_path():
    """Return the path of a test file as a pathlib.Path."""
    yield ISOCHRONES_WALK


@pytest.fixture
def file_digest_test_file_as_str(file_digest_test_file_as_pathlib_path):
    """Return the path of a test file as a str."""
    yield f"{file_digest_test_file_as_pathlib_path}"


@pytest.fixture()
def file_digest_sha256():
    """Return the expected SHA256 hash for the test file."""
    yield ISOCHRONES_WALK_SHA256


@pytest.fixture()
def file_digest_blake2b():
    """Return the expected BLAKE2B hash for the test file."""
    yield ISOCHRONES_WALK_BLAKE2B


@pytest.fixture()
def file_digest_blake2s():
    """Return the expected BLAKE2S hash for the test file."""
    yield ISOCHRONES_WALK_BLAKE2S



#!/usr/bin/env python3


"""Wraps a com.conveyal.r5.transit.TransitLayer."""


import functools

import jpype
import jpype.types

import java.time


__all__ = ["TransitLayer"]


class TransitLayer:
    """Wrap a com.conveyal.r5.transit.TransitLayer."""

    @classmethod
    def from_r5_transit_layer(cls, transit_layer):
        """
        Create a TransitLayer from a com.conveyal.r5.transit.TransitLayer.

        Arguments
        ---------
        transit_layer : com.conveyal.r5.transit.TransitLayer
        """
        instance = cls()
        instance._transit_layer = transit_layer
        return instance

    def covers(self, date):
        """
        Check whether `date` is covered by GTFS data sets.

        Arguments:
        ----------
        date : datetime.date
            date for which to check whether a GTFS service exists.

        Returns:
        --------
        bool
            Whether or not any services exist on `date`.
        """
        date = java.time.LocalDate.of(date.year, date.month, date.day)
        return True in set(
            [service.activeOn(date) for service in self._transit_layer.services]
        )

    def get_street_vertex_for_stop(self, stop):
        """
        Get the street layer’s vertex corresponding to `stop`.

        Arguments
        ---------
        stop : int
            ID of the public transport stop for which to find a vertex

        Returns
        -------
        int
            ID of the vertex corresponding to the public transport stop
        """
        street_vertex = self._transit_layer.streetVertexForStop.get(stop)
        return street_vertex

    @functools.cached_property
    def routes(self):
        """Return a list of GTFS routes."""
        return list(self._transit_layer.routes)

    @functools.cached_property
    def trip_patterns(self):
        """Return a list of GTFS trip patterns."""
        return list(self._transit_layer.tripPatterns)

    def get_stop_id_from_index(self, stop_index):
        """Get the GTFS stop id for the `stop_index`-th stop of this transit layer."""
        return self._transit_layer.stopIdForIndex[stop_index]


@jpype._jcustomizer.JConversion(
    "com.conveyal.r5.transit.TransitLayer", exact=TransitLayer
)
def _cast_TransitLayer(java_class, object_):
    return object_._transit_layer



#!/usr/bin/env python3


import pathlib

import r5py

import com.conveyal.file
import java.io


class TestFileStorage:
    def test_void_methods(self):
        file_storage = r5py.r5.file_storage.FileStorage()
        assert file_storage.moveIntoStorage() is None
        assert file_storage.delete() is None

    def test_get_file(self):
        file_storage = r5py.r5.file_storage.FileStorage()
        fake_file_path = pathlib.Path("/home/foo/bar.txt")
        gotten_file = file_storage.getFile(
            com.conveyal.file.FileStorageKey(
                com.conveyal.file.FileCategory.DATASOURCES,
                f"{fake_file_path.with_suffix('')}",
            )
        )
        assert isinstance(gotten_file, java.io.File)
        assert str(gotten_file) == f"{fake_file_path.with_suffix('')}"

    def test_get_url(self):
        file_storage = r5py.r5.file_storage.FileStorage()
        fake_file_path = pathlib.Path("/home/foo/bar.txt")
        gotten_url = file_storage.getURL(
            com.conveyal.file.FileStorageKey(
                com.conveyal.file.FileCategory.DATASOURCES,
                f"{fake_file_path.with_suffix('')}",
            )
        )
        assert isinstance(gotten_url, str)
        assert gotten_url == f"file://{fake_file_path.with_suffix('')}"

    def test_exists(self):
        file_storage = r5py.r5.file_storage.FileStorage()
        fake_file_path = pathlib.Path("/home/foo/bar.txt")
        exists = file_storage.exists(
            com.conveyal.file.FileStorageKey(
                com.conveyal.file.FileCategory.DATASOURCES,
                f"{fake_file_path.with_suffix('')}",
            )
        )
        assert not exists



#!/usr/bin/env python3


"""A thin layer around com.conveyal.r5.file.FileStorage."""


import jpype

from ..util import start_jvm

import com.conveyal.file
import java.io.File


__all__ = ["FileStorage"]


start_jvm()


@jpype.JImplements(com.conveyal.file.FileStorage)
class FileStorage:
    """A thin layer around com.conveyal.r5.file.FileStorage."""

    @jpype.JOverride
    def moveIntoStorage(self, *args):
        """Not implemented."""
        pass

    @jpype.JOverride
    def getFile(self, file_storage_key):
        """
        Return a java.io.File for the file identified by file_storage_key.

        Arguments
        ---------
        file_storage_key : com.conveyal.r5.file.FileStorageKey
            An R5 object referencing a certain file

        Returns
        -------
        java.io.File
            The file identified by file_storage_key
        """
        return java.io.File(file_storage_key.path)

    @jpype.JOverride
    def getURL(self, file_storage_key):
        """
        Return an URL for the file identified by file_storage_key.

        Arguments
        ---------
        file_storage_key : com.conveyal.r5.file.FileStorageKey
            An R5 object referencing a certain file

        Returns
        -------
        str
            An imaginary URL pointing to the file identified by file_storage_key
        """
        return f"file://{file_storage_key.path}"

    @jpype.JOverride
    def delete(self, *args):
        """Not implemented."""
        pass

    @jpype.JOverride
    def exists(self, file_storage_key):
        """
        Check whether the file identified by file_storage_key exists.

        Arguments
        ---------
        file_storage_key : com.conveyal.r5.file.FileStorageKey
            An R5 object referencing a certain file

        Returns
        -------
        bool
            Whether or not the file identified by file_storage_key exists
        """
        return self.getFile(file_storage_key).exists()
